       ### Clear Workspace ###
rm(list=ls(all=TRUE))
zeit1<-Sys.time()

### Load Data ###
# set working directory with setwd()
data <- read.table("AR(2).txt")

# Number of observations
n <-1000 # 100, 250, 500, 1000

# Number of Replications
Q <- ncol(data)

 ### maximum estimated model order
 p.max<-2
 p.row1<-p.max^2  # two regime models
 p.row<-p.max^3   # three regime models
 P1<-cbind(rep(1:p.max,each=p.max),rep(1:p.max,p.max))
 P<-cbind(rep(1:p.max,each=p.max^2),rep(rep(1:p.max,each=p.max),p.max),rep(1:p.max,p.max^2))

 # Decision Matrices
IC<-array(NA,c(Q,24,p.row))    #row=Replication, col=Criterium, matrix=Model 

for(q in 1:Q)
{
 y <- data[(1:n),q]
 # effective sample size
 y.eff <- y[(p.max+1):n]

### SETAR Model with 3 Regimes ###

# additional parameters: 2 threshold, 1 delay
R.tar <- 3

#grid for threshold and delay parameter
kc <- seq(quantile(y,0.1),quantile(y,0.9),length=9)
kd <- 1:p.max

#lagged variables
lag.tar <-matrix(1:p.max,nrow=(n-p.max),ncol=p.max,byrow=TRUE)
Lag.tar <-as.vector(((p.max+1):n)-lag.tar)
x.tar   <- y[Lag.tar]
x.tar   <- matrix(x.tar,ncol=p.max)
T.r1    <- numeric(length(kc))
T.r2    <- numeric(length(kc))
T.r12   <- matrix(NA, nrow=p.row, ncol=length(kc)-1)
T.r22   <- matrix(NA, nrow=p.row, ncol=length(kc)-1)
T.r32   <- matrix(NA, nrow=p.row, ncol=length(kc)-1)
RSS.r1  <- array(NA,c(length(kc),p.max,p.row)) #row=threshold ,col=delay ,matrix=model
RSS.r2  <- array(NA,c(length(kc),p.max,p.row)) #row=threshold ,col=delay ,matrix=model
RSS.r12 <- matrix(NA, nrow=p.row, ncol=length(kc)-1)
RSS.r22 <- matrix(NA, nrow=p.row, ncol=length(kc)-1)
RSS.r32 <- matrix(NA, nrow=p.row, ncol=length(kc)-1)
RSS     <- array(NA,c(length(kc),p.max,p.row)) #row=threshold ,col=delay ,matrix=model
par.hat <- matrix(NA, nrow=p.row,ncol=2)       #1st col="threshold", 2nd col=delay
W       <- array(NA,c((length(kc)-1),(n-p.max),p.row1))  #row=2nd threshold, col=weights, matrix=model(2reg)
thresh2 <- numeric(p.row)
sigma.r1<- matrix(NA,nrow=2,ncol=p.row)
sigma.r2<- matrix(NA,nrow=2,ncol=p.row)
sigma.r3<- matrix(NA,nrow=2,ncol=p.row)
sigma   <- matrix(NA,nrow=2,ncol=p.row)

for(d in 1:p.max){
 for(k in 1:length(kc)){
  w <- numeric(n)
  for(i in (p.max+1):n){
   if(y[i-d]>kc[k]){w[i]<-1}
   }
   w <- w[(p.max+1):n]
   y.r1 <- y.eff[w==1]
   y.r2 <- y.eff[w==0]
   x.r1 <- x.tar[w==1,]
   x.r2 <- x.tar[w==0,]
   T.r1[k] <- length(y.r1)
   T.r2[k] <- length(y.r2)
   for(p in 1:p.row1){
    RSS.r1[k,d,p]<-sum(summary(lm(y.r1~x.r1[,(1:P1[p,1])]))$resid^2)
    RSS.r2[k,d,p]<-sum(summary(lm(y.r2~x.r2[,(1:P1[p,2])]))$resid^2)
  }
 }
}

RSS <- RSS.r1 + RSS.r2      #row=threshold ,col=delay ,matrix=model

# estimated threshold and delay parameter for SETAR models
for(p in 1:p.row1){
 if(nrow(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE))==1){
 par.hat[p,] <- which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)      #1st col="threshold", 2nd col=delay
 }else{
 par.hat[p,] <- c(round(mean(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)[,1])),min(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)[,2]))
}
}
 
kc.new <-  matrix(NA, ncol=(length(kc)-1),nrow=p.row1)
for(p in 1:p.row1)
{
 kc.new[p,] <- kc[-par.hat[p,1]]
 
 for(k in 1:ncol(kc.new))
 {
  w <- numeric(n)
  for(i in (p.max+1):n){
  if(kc[par.hat[p,1]]>kc.new[p,k]){ 
   if(y[i-par.hat[p,2]]<kc.new[p,k]){w[i]<-3}else{w[i]<-2}
   if(y[i-par.hat[p,2]]>kc[par.hat[p,1]]){w[i]<-1}
  }else{
   if(y[i-par.hat[p,2]]<kc.new[p,k]){w[i]<-2}else{w[i]<-1}
   if(y[i-par.hat[p,2]]<kc[par.hat[p,1]]){w[i]<-3}
 }
 }
 W[k,,p] <-  w[(p.max+1):n]
}
} 


for(k in 1:ncol(kc.new))
{
 for(p in 1:p.row)
 {
  if(p==1 | p==2){p.new <- 1} 
  if(p==3 | p==4){p.new <- 2}
  if(p==5 | p==6){p.new <- 3}
  if(p==7 | p==8){p.new <- 4}
  y.r1 <- y.eff[W[k,,p.new]==1]
  y.r2 <- y.eff[W[k,,p.new]==2]
  y.r3 <- y.eff[W[k,,p.new]==3]
  x.r1 <- x.tar[W[k,,p.new]==1,]
  x.r2 <- x.tar[W[k,,p.new]==2,]
  x.r3 <- x.tar[W[k,,p.new]==3,]
  T.r12[p,k] <- length(y.r1)
  T.r22[p,k] <- length(y.r2)
  T.r32[p,k] <- length(y.r3)
   if(T.r12[p,k]>=2){RSS.r12[p,k] <- sum(summary(lm(y.r1 ~ x.r1[,(1:P[p,1])]))$resid^2)}else{RSS.r12[p,k]<-0}
   if(T.r22[p,k]>=2){RSS.r22[p,k] <- sum(summary(lm(y.r2 ~ x.r2[,(1:P[p,2])]))$resid^2)}else{RSS.r22[p,k]<-0}
   if(T.r32[p,k]>=2){RSS.r32[p,k] <- sum(summary(lm(y.r3 ~ x.r3[,(1:P[p,3])]))$resid^2)}else{RSS.r32[p,k]<-0}
  }
 }
 
 RSS.total <- RSS.r12+RSS.r22+RSS.r32
  T.rfin <- matrix(NA,nrow=p.row, ncol=3)
 for(p in 1:p.row)
 {
  thresh2[p] <- which(RSS.total[p,]==min(RSS.total[p,]),arr.ind=TRUE) 
  T.rfin[p,] <- c(T.r12[p,thresh2[p]],T.r22[p,thresh2[p]],T.r32[p,thresh2[p]])  

# estimates error term variances for SETAR models
sigma.r1[1,p] <- RSS.r12[p,thresh2[p]]/(T.rfin[p,1])
sigma.r1[2,p] <- RSS.r12[p,thresh2[p]]/(T.rfin[p,1]-P[p,1]-1)
sigma.r2[1,p] <- RSS.r22[p,thresh2[p]]/(T.rfin[p,2])
sigma.r2[2,p] <- RSS.r22[p,thresh2[p]]/(T.rfin[p,2]-P[p,2]-1)
sigma.r3[1,p] <- RSS.r32[p,thresh2[p]]/(T.rfin[p,3])
sigma.r3[2,p] <- RSS.r32[p,thresh2[p]]/(T.rfin[p,3]-P[p,3]-1)
sigma[1,p]    <- RSS.total[p,thresh2[p]]/(n-p.max)
sigma[2,p]    <- RSS.total[p,thresh2[p]]/(n-p.max-P[p,1]-P[p,2]-P[p,3]-1)


    ### calculate information criteria of SETAR models ###
    # regime 1 criteria
    AIC.tar.r1.ML <- T.rfin[p,1]*(log(sigma.r1[1,p])+1)+2*(P[p,1]+1+R.tar)
    AIC.tar.r1.LS <- T.rfin[p,1]*(log(sigma.r1[2,p])+1)+2*(P[p,1]+1+R.tar)
    BIC.tar.r1.ML <- T.rfin[p,1]*log(sigma.r1[1,p])+(P[p,1]+R.tar)*log(T.rfin[p,1])
    BIC.tar.r1.LS <- T.rfin[p,1]*log(sigma.r1[2,p])+(P[p,1]+R.tar)*log(T.rfin[p,1])
    AICc.tar.r1.ML<- T.rfin[p,1]*log(sigma.r1[1,p])+(T.rfin[p,1]+P[p,1]+R.tar)/(1-(P[p,1]+2+R.tar)/T.rfin[p,1])
    AICc.tar.r1.LS<- T.rfin[p,1]*log(sigma.r1[2,p])+(T.rfin[p,1]+P[p,1]+R.tar)/(1-(P[p,1]+2+R.tar)/T.rfin[p,1])
    WIC.tar.r1.ML <- T.rfin[p,1]*log(sigma.r1[1,p])+((2*T.rfin[p,1]*(P[p,1]+1+R.tar)/(T.rfin[p,1]-P[p,1]-2-R.tar))^2+((P[p,1]+R.tar)*log(T.rfin[p,1]))^2)/((2*T.rfin[p,1]*(P[p,1]+1+R.tar)/(T.rfin[p,1]-P[p,1]-2-R.tar))+((P[p,1]+R.tar)*log(T.rfin[p,1])))
    WIC.tar.r1.LS <- T.rfin[p,1]*log(sigma.r1[2,p])+((2*T.rfin[p,1]*(P[p,1]+1+R.tar)/(T.rfin[p,1]-P[p,1]-2-R.tar))^2+((P[p,1]+R.tar)*log(T.rfin[p,1]))^2)/((2*T.rfin[p,1]*(P[p,1]+1+R.tar)/(T.rfin[p,1]-P[p,1]-2-R.tar))+((P[p,1]+R.tar)*log(T.rfin[p,1])))

    # regime 2 criteria
    AIC.tar.r2.ML <- T.rfin[p,2]*(log(sigma.r2[1,p])+1)+2*(P[p,2]+1+R.tar)
    AIC.tar.r2.LS <- T.rfin[p,2]*(log(sigma.r2[2,p])+1)+2*(P[p,2]+1+R.tar)
    BIC.tar.r2.ML <- T.rfin[p,2]*log(sigma.r2[1,p])+(P[p,2]+R.tar)*log(T.rfin[p,2])
    BIC.tar.r2.LS <- T.rfin[p,2]*log(sigma.r2[2,p])+(P[p,2]+R.tar)*log(T.rfin[p,2])
    AICc.tar.r2.ML<- T.rfin[p,2]*log(sigma.r2[1,p])+(T.rfin[p,2]+P[p,2]+R.tar)/(1-(P[p,2]+2+R.tar)/T.rfin[p,2])
    AICc.tar.r2.LS<- T.rfin[p,2]*log(sigma.r2[2,p])+(T.rfin[p,2]+P[p,2]+R.tar)/(1-(P[p,2]+2+R.tar)/T.rfin[p,2])
    WIC.tar.r2.ML <- T.rfin[p,2]*log(sigma.r2[1,p])+((2*T.rfin[p,2]*(P[p,2]+1+R.tar)/(T.rfin[p,2]-P[p,2]-2-R.tar))^2+((P[p,2]+R.tar)*log(T.rfin[p,2]))^2)/((2*T.rfin[p,2]*(P[p,2]+1+R.tar)/(T.rfin[p,2]-P[p,2]-2-R.tar))+((P[p,2]+R.tar)*log(T.rfin[p,2])))
    WIC.tar.r2.LS <- T.rfin[p,2]*log(sigma.r2[2,p])+((2*T.rfin[p,2]*(P[p,2]+1+R.tar)/(T.rfin[p,2]-P[p,2]-2-R.tar))^2+((P[p,2]+R.tar)*log(T.rfin[p,2]))^2)/((2*T.rfin[p,2]*(P[p,2]+1+R.tar)/(T.rfin[p,2]-P[p,2]-2-R.tar))+((P[p,2]+R.tar)*log(T.rfin[p,2])))

    # regime 3 criteria
    AIC.tar.r3.ML <- T.rfin[p,3]*(log(sigma.r3[1,p])+1)+2*(P[p,3]+1+R.tar)
    AIC.tar.r3.LS <- T.rfin[p,3]*(log(sigma.r3[2,p])+1)+2*(P[p,3]+1+R.tar)
    BIC.tar.r3.ML <- T.rfin[p,3]*log(sigma.r3[1,p])+(P[p,3]+R.tar)*log(T.rfin[p,3])
    BIC.tar.r3.LS <- T.rfin[p,3]*log(sigma.r3[2,p])+(P[p,3]+R.tar)*log(T.rfin[p,3])
    AICc.tar.r3.ML<- T.rfin[p,3]*log(sigma.r3[1,p])+(T.rfin[p,3]+P[p,3]+R.tar)/(1-(P[p,3]+2+R.tar)/T.rfin[p,3])
    AICc.tar.r3.LS<- T.rfin[p,3]*log(sigma.r3[2,p])+(T.rfin[p,3]+P[p,3]+R.tar)/(1-(P[p,3]+2+R.tar)/T.rfin[p,3])
    WIC.tar.r3.ML <- T.rfin[p,3]*log(sigma.r3[1,p])+((2*T.rfin[p,3]*(P[p,3]+1+R.tar)/(T.rfin[p,3]-P[p,3]-2-R.tar))^2+((P[p,3]+R.tar)*log(T.rfin[p,3]))^2)/((2*T.rfin[p,3]*(P[p,3]+1+R.tar)/(T.rfin[p,3]-P[p,3]-2-R.tar))+((P[p,3]+R.tar)*log(T.rfin[p,3])))
    WIC.tar.r3.LS <- T.rfin[p,3]*log(sigma.r3[2,p])+((2*T.rfin[p,3]*(P[p,3]+1+R.tar)/(T.rfin[p,3]-P[p,3]-2-R.tar))^2+((P[p,3]+R.tar)*log(T.rfin[p,3]))^2)/((2*T.rfin[p,3]*(P[p,3]+1+R.tar)/(T.rfin[p,3]-P[p,3]-2-R.tar))+((P[p,3]+R.tar)*log(T.rfin[p,3])))

    # overall criteria
    AIC.tar.ML.ov <- (n-p.max)*(log(sigma[1,p])+1)+2*(P[p,1]+P[p,2]+P[p,3]+1+R.tar)
    AIC.tar.LS.ov <- (n-p.max)*(log(sigma[2,p])+1)+2*(P[p,1]+P[p,2]+P[p,3]+1+R.tar)
    BIC.tar.ML.ov <- (n-p.max)*log(sigma[1,p])+(P[p,1]+P[p,2]+P[p,3]+R.tar)*log(n-p.max)
    BIC.tar.LS.ov <- (n-p.max)*log(sigma[2,p])+(P[p,1]+P[p,2]+P[p,3]+R.tar)*log(n-p.max)
    AICc.tar.ML.ov<- (n-p.max)*log(sigma[1,p])+(n-p.max+(P[p,1]+P[p,2]+P[p,3]+R.tar))/(1-((P[p,1]+P[p,2]+P[p,3]+R.tar)+2)/(n-p.max))
    AICc.tar.LS.ov<- (n-p.max)*log(sigma[2,p])+(n-p.max+(P[p,1]+P[p,2]+P[p,3]+R.tar))/(1-((P[p,1]+P[p,2]+P[p,3]+R.tar)+2)/(n-p.max))
    WIC.tar.ML.ov <- (n-p.max)*log(sigma[1,p])+((2*(n-p.max)*((P[p,1]+P[p,2]+P[p,3]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+P[p,3]+R.tar)-2))^2+((P[p,1]+P[p,2]+P[p,3]+R.tar)*log(n-p.max))^2)/((2*(n-p.max)*((P[p,1]+P[p,2]+P[p,3]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+P[p,3]+R.tar)-2))+((P[p,1]+P[p,2]+P[p,3]+R.tar)*log(n-p.max)))
    WIC.tar.LS.ov <- (n-p.max)*log(sigma[2,p])+((2*(n-p.max)*((P[p,1]+P[p,2]+P[p,3]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+P[p,3]+R.tar)-2))^2+((P[p,1]+P[p,2]+P[p,3]+R.tar)*log(n-p.max))^2)/((2*(n-p.max)*((P[p,1]+P[p,2]+P[p,3]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+P[p,3]+R.tar)-2))+((P[p,1]+P[p,2]+P[p,3]+R.tar)*log(n-p.max)))
####
    ### tabulate information criteria ###
    IC[q,1,p]<-T.rfin[p,1]/(n-p.max)*AIC.tar.r1.ML+T.rfin[p,2]/(n-p.max)*AIC.tar.r2.ML+T.rfin[p,3]/(n-p.max)*AIC.tar.r3.ML
    IC[q,2,p]<-T.rfin[p,1]/(n-p.max)*AIC.tar.r1.LS+T.rfin[p,2]/(n-p.max)*AIC.tar.r2.LS+T.rfin[p,3]/(n-p.max)*AIC.tar.r3.LS
    IC[q,3,p]<-T.rfin[p,1]/(n-p.max)*BIC.tar.r1.ML+T.rfin[p,2]/(n-p.max)*BIC.tar.r2.ML+T.rfin[p,3]/(n-p.max)*BIC.tar.r3.ML
    IC[q,4,p]<-T.rfin[p,1]/(n-p.max)*BIC.tar.r1.LS+T.rfin[p,2]/(n-p.max)*BIC.tar.r2.LS+T.rfin[p,3]/(n-p.max)*BIC.tar.r3.LS
    IC[q,5,p]<-T.rfin[p,1]/(n-p.max)*AICc.tar.r1.ML+T.rfin[p,2]/(n-p.max)*AICc.tar.r2.ML+T.rfin[p,3]/(n-p.max)*AICc.tar.r3.ML
    IC[q,6,p]<-T.rfin[p,1]/(n-p.max)*AICc.tar.r1.LS+T.rfin[p,2]/(n-p.max)*AICc.tar.r2.LS+T.rfin[p,3]/(n-p.max)*AICc.tar.r3.LS
    IC[q,7,p]<-T.rfin[p,1]/(n-p.max)*WIC.tar.r1.ML+T.rfin[p,2]/(n-p.max)*WIC.tar.r2.ML+T.rfin[p,3]/(n-p.max)*WIC.tar.r3.ML
    IC[q,8,p]<-T.rfin[p,1]/(n-p.max)*WIC.tar.r1.LS+T.rfin[p,2]/(n-p.max)*WIC.tar.r2.LS+T.rfin[p,3]/(n-p.max)*WIC.tar.r3.LS
    IC[q,9,p]<-(AIC.tar.r1.ML+AIC.tar.r2.ML+AIC.tar.r3.ML)/3
    IC[q,10,p]<-(AIC.tar.r1.LS+AIC.tar.r2.LS+AIC.tar.r3.LS)/3
    IC[q,11,p]<-(BIC.tar.r1.ML+BIC.tar.r2.ML+BIC.tar.r3.ML)/3
    IC[q,12,p]<-(BIC.tar.r1.LS+BIC.tar.r2.LS+BIC.tar.r3.LS)/3
    IC[q,13,p]<-(AICc.tar.r1.ML+AICc.tar.r2.ML+AICc.tar.r3.ML)/3
    IC[q,14,p]<-(AICc.tar.r1.LS+AICc.tar.r2.LS+AICc.tar.r3.LS)/3
    IC[q,15,p]<-(WIC.tar.r1.ML+WIC.tar.r2.ML+WIC.tar.r3.ML)/3
    IC[q,16,p]<-(WIC.tar.r1.LS+WIC.tar.r2.LS+WIC.tar.r3.LS)/3
    IC[q,17,p]<-AIC.tar.ML.ov
    IC[q,18,p]<-AIC.tar.LS.ov
    IC[q,19,p]<-BIC.tar.ML.ov
    IC[q,20,p]<-BIC.tar.LS.ov
    IC[q,21,p]<-AICc.tar.ML.ov
    IC[q,22,p]<-AICc.tar.LS.ov
    IC[q,23,p]<-WIC.tar.ML.ov
    IC[q,24,p]<-WIC.tar.LS.ov
}
print(q)
}    

# write and save IC for each model
for (i in 1:p.row) {
    a <- data.frame(IC[, ,i])
    myfile <- file.path(paste0("IC.1000.AR(2)1step.TAR", i, ".xls"))
    write.table(a, file = myfile, sep = '\t', row.names = FALSE, col.names = FALSE, quote = FALSE, append = FALSE)
}


zeit2<-Sys.time()
zeit2-zeit1


