### Clear Workspace ###
rm(list=ls(all=TRUE))
zeit1<-Sys.time()

### Load Data ###
# set working directory with setwd()
data <- read.table("AR(2).txt")

# Number of observations
n <- 1000 # 100, 250, 500, 1000

# Number of Replications
Q <- ncol(data)

 ### maximum estimated model order
 p.max<-4 # 2 for estimating number of regimes
 p.row<-p.max^2
 P<-cbind(rep(1:p.max,each=p.max),rep(1:p.max,p.max))

 # Decision Matrices
IC<-array(NA,c(Q,24,p.max))    #row=Replication, col=Criterium, matrix=Model (AR(1),AR(2),...)

for(q in 1:Q)
{
 y <- data[(1:n),q]
 # effective sample size
 y.eff <- y[(p.max+1):n]

### AR Model ###

# lagged variables
lag.ar <- matrix(1:p.max,nrow=(n-p.max),ncol=p.max,byrow=TRUE)
Lag.ar <- as.vector(((p.max+1):n)-lag.ar)
x.ar   <- y[Lag.ar]
x.ar   <- matrix(x.ar, ncol=p.max)
RSS.ar<-rep(NA,p.max)
sigma.AR<-matrix(NA, nrow=2, ncol=p.max)  #1st row= ML estimates (hat sigma), 2nd row= LS estimates (tilde sigma), col= models

# AR(p)
for(p in 1:p.max){
RSS.ar[p]<-assign(paste("RSS",p,sep=""),sum(summary(lm(y.eff~x.ar[,(1:p)]))$resid^2))


### error term variance estimates of linear models ###
                                                   #col= models
 sigma.AR[1,p] <- RSS.ar[p]/(n-p.max)              #1st row= ML estimates (hat sigma),
 sigma.AR[2,p] <- RSS.ar[p]/(n-p.max-p-1)          #2nd row= LS estimates (tilde sigma),


    ### calculate information criteria of linear models ###
    AIC.ar.ML<-(n-p.max)*(log(sigma.AR[1,p])+1)+2*(p+1)
    AIC.ar.OLS<-(n-p.max)*(log(sigma.AR[2,p])+1)+2*(p+1)
    BIC.ar.ML<-(n-p.max)*log(sigma.AR[1,p])+p*log(n-p.max)
    BIC.ar.OLS<-(n-p.max)*log(sigma.AR[2,p])+p*log(n-p.max)
    AICc.ar.ML<-(n-p.max)*log(sigma.AR[1,p])+(n-p.max+p)/(1-(p+2)/(n-p.max))
    AICc.ar.OLS<-(n-p.max)*log(sigma.AR[2,p])+(n-p.max+p)/(1-(p+2)/(n-p.max))
    WIC.ar.ML <- (n-p.max)*log(sigma.AR[1,p])+((2*(n-p.max)*(p+1)/((n-p.max)-p-2))^2+(p*log(n-p.max))^2)/((2*(n-p.max)*(p+1)/((n-p.max)-p-2))+(p*log(n-p.max)))
    WIC.ar.OLS <- (n-p.max)*log(sigma.AR[2,p])+((2*(n-p.max)*(p+1)/((n-p.max)-p-2))^2+(p*log(n-p.max))^2)/((2*(n-p.max)*(p+1)/((n-p.max)-p-2))+(p*log(n-p.max)))

    ### tabulate information criteria ###
    IC[q,1,p]<-AIC.ar.ML     #(hat sigma)
    IC[q,2,p]<-AIC.ar.OLS    #(tilde sigma)
    IC[q,3,p]<-BIC.ar.ML
    IC[q,4,p]<-BIC.ar.OLS
    IC[q,5,p]<-AICc.ar.ML
    IC[q,6,p]<-AICc.ar.OLS
    IC[q,7,p]<-WIC.ar.ML
    IC[q,8,p]<-WIC.ar.OLS
    IC[q,9,p]<-AIC.ar.ML
    IC[q,10,p]<-AIC.ar.OLS
    IC[q,11,p]<-BIC.ar.ML
    IC[q,12,p]<-BIC.ar.OLS
    IC[q,13,p]<-AICc.ar.ML
    IC[q,14,p]<-AICc.ar.OLS
    IC[q,15,p]<-WIC.ar.ML
    IC[q,16,p]<-WIC.ar.OLS
    IC[q,17,p]<-AIC.ar.ML
    IC[q,18,p]<-AIC.ar.OLS
    IC[q,19,p]<-BIC.ar.ML
    IC[q,20,p]<-BIC.ar.OLS
    IC[q,21,p]<-AICc.ar.ML
    IC[q,22,p]<-AICc.ar.OLS
    IC[q,23,p]<-WIC.ar.ML
    IC[q,24,p]<-WIC.ar.OLS
}    
print(q)
}    

# write and save IC for each model
for (i in 1:p.max) {
    a <- data.frame(IC[, ,i])
    myfile <- file.path(paste0("IC.1000.AR(2).AR", i, ".xls"))
    write.table(a, file = myfile, sep = '\t', row.names = FALSE, col.names = FALSE, quote = FALSE, append = FALSE)
}


zeit2<-Sys.time()
zeit2-zeit1



