### Clear Workspace ###
rm(list=ls(all=TRUE))
zeit1<-Sys.time()

### Load Data ###
# set working directory with setwd()
data <- read.table("AR(2).txt")

# Number of observations
n <-1000 # 100, 250, 500, 1000

# Number of Replications
Q <- ncol(data)

 ### maximum estimated model order
 p.max<-4
 p.row<-p.max^2
 P<-cbind(rep(1:p.max,each=p.max),rep(1:p.max,p.max))

 # Decision Matrices
IC<-array(NA,c(Q,24,p.row))    #row=Replication, col=Criterium, matrix=Model

for(q in 1:Q)
{
 y <- data[(1:n),q]
 # effective sample size
 y.eff <- y[(p.max+1):n]

 ### ESTAR Model ###

# additional parameters: threshold, delay, transition gamma
R.star <- 3

#grid for threshold and delay parameter
kc <- seq(quantile(y,0.15),quantile(y,0.85),length=15)
kd <- 1:p.max

# lagged variables
lag.estar <- matrix(1:p.max,nrow=(n-p.max),ncol=p.max,byrow=TRUE)
Lag.estar <- as.vector(((p.max+1):n)-lag.estar)
x.estar   <- y[Lag.estar]
x.estar   <- matrix(x.estar, ncol=p.max)


RSS       <- array(NA,c(length(kc),p.max,p.row)) #row=threshold ,col=delay ,matrix=model
par.hat   <- matrix(NA, nrow=p.row,ncol=2)       #1st col="threshold", 2nd col=delay
beta.hat  <- NULL
RSS.opt   <- numeric(p.row)
gamma.opt <- numeric(p.row)
weight    <- matrix(0, ncol=p.row, nrow=n-p.max)
Trans.func <- matrix(NA,ncol=p.row,nrow=n-p.max)
T.r1      <- numeric(p.row)
T.r2      <- numeric(p.row)
Res       <- matrix(NA,ncol=p.row,nrow=n-p.max)
RSS.r1    <- numeric(p.row)
RSS.r2    <- numeric(p.row)
sigma     <- matrix(NA,nrow=2,ncol=p.row)
sigma.r1  <- matrix(NA,nrow=2,ncol=p.row)
sigma.r2  <- matrix(NA,nrow=2,ncol=p.row)



# calculate RSS for all possible threshold-delay-model combinations
for(d in 1:p.max){
 for(k in 1:length(kc)){
  trans.func <- numeric(n)
  for(i in (p.max+1):n){
   trans.func[i]<-1-exp(-1*(y[i-d]-kc[k])^2/var(y.eff))            #starting value gamma=1
  }
  trans.func <- trans.func[(p.max+1):n]
  x.trans <- trans.func*x.estar
  for(p in 1:p.row){
   RSS[k,d,p] <- sum(summary(lm(y.eff ~ x.estar[,1:P[p,1]] + trans.func + x.trans[,1:P[p,2]]))$resid^2)
  }
 }
}
   
# estimated threshold and delay parameter for ESTAR models
for(p in 1:p.row){
 if(nrow(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE))==1){
 par.hat[p,] <- which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)      #1st col="threshold", 2nd col=delay
 }else{
 par.hat[p,] <- c(round(mean(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)[,1])),min(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)[,2]))
}
}

# estimated beta coefficients (persistence parameters)
 for(p in 1:p.row){
  trans.func <- numeric(n)
  for(i in (p.max+1):n){
   trans.func[i]<-1-exp(-1*(y[i-par.hat[p,2]]-kc[par.hat[p,1]])^2/var(y.eff))            #starting value gamma=1
  }
  trans.func <- trans.func[(p.max+1):n]
  x.trans <- trans.func*x.estar
   beta.hat[[p]] <- assign(paste("beta.hat",p,sep=""), lm(y.eff ~ x.estar[,1:P[p,1]] + trans.func + x.trans[,1:P[p,2]])$coef)
 }

# RSS as a function of gamma
RSS.func <- function(gam,y.eff,y,p,x.estar,par.hat,kc)
{
 regressor <- matrix(1,nrow=n,ncol=(P[p,1]+P[p,2]+2))
 regressor[(p.max+1):n,(2:(P[p,1]+1))] <- x.estar[,1:P[p,1]]
 for(i in (p.max+1):n){
 regressor[i,(P[p,1]+2)] <- 1-exp(-gam*(y[i-par.hat[p,2]]-kc[par.hat[p,1]])^2/var(y.eff))
 regressor[i,(P[p,1]+3):(P[p,1]+P[p,2]+2)] <- (1-exp(-gam*(y[i-par.hat[p,2]]-kc[par.hat[p,1]])^2/var(y.eff)))*x.estar[i-p.max,1:P[p,2]]
 }
 regressor <- regressor[(p.max+1):n,]
 rss <- sum((y.eff-regressor%*%beta.hat[[p]])^2)
 return(rss)
}

# residual function
res.func <- function(y.eff,p,x.estar,trans)
{
 regressor <- matrix(1,nrow=n,ncol=(P[p,1]+P[p,2]+2))
 regressor[(p.max+1):n,(2:(P[p,1]+1))] <- x.estar[,1:P[p,1]]
 for(i in (p.max+1):n){
 regressor[i,(P[p,1]+2)] <- trans[i-p.max,p]
 regressor[i,(P[p,1]+3):(P[p,1]+P[p,2]+2)] <- trans[i-p.max,p]*x.estar[i-p.max,1:P[p,2]]
 }
 regressor <- regressor[(p.max+1):n,]
 res <- y.eff-regressor%*%beta.hat[[p]]
 return(res)
}

for(p in 1:p.row){
# find optimal gamma and respective RSS
opt <- optim(par=c(1),fn=RSS.func, #method=c("Brent"),
 y.eff=y.eff,y=y,p=p,x.estar=x.estar,par.hat=par.hat,kc=kc)
RSS.opt[p] <- opt$value
gamma.opt[p] <- opt$par
for(i in (p.max+1):n){
# separate regimes
if(y[i-par.hat[p,2]]>kc[par.hat[p,1]]){weight[i-p.max,p]<-1}
# calculate transition function
Trans.func[(i-p.max),p] <- 1-exp(-gamma.opt[p]*(y[i-par.hat[p,2]]-kc[par.hat[p,1]])^2/var(y.eff))
}
# number of observations in each regime
T.r1[p] <- sum(weight[,p])
T.r2[p] <- n-p.max-T.r1[p]
# calculate residuals in each regime and respective RSS
Res[,p] <- res.func(y.eff=y.eff,p=p,x.estar=x.estar,trans=Trans.func)
RSS.r1[p] <- sum(Res[(weight[,p]==1),p]^2)
RSS.r2[p] <- sum(Res[(weight[,p]==0),p]^2)
# calculate estimated error term variance for whole model and single regimes
sigma[1,p]<- RSS.opt[p]/(n-p.max)
sigma[2,p]<- RSS.opt[p]/(n-p.max-P[p,1]-P[p,2]-1)
sigma.r1[1,p]<- RSS.r1[p]/T.r1[p]
sigma.r1[2,p]<- RSS.r1[p]/(T.r1[p]-P[p,1]-1)
sigma.r2[1,p]<- RSS.r2[p]/T.r2[p]
sigma.r2[2,p]<- RSS.r2[p]/(T.r2[p]-P[p,2]-1)


    ### calculate information criteria of ESTAR models ###
    # regime 1 criteria
    AIC.estar.r1.ML<-T.r1[p]*(log(sigma.r1[1,p])+1)+2*(P[p,1]+1+R.star)
    AIC.estar.r1.LS<-T.r1[p]*(log(sigma.r1[2,p])+1)+2*(P[p,1]+1+R.star)
    BIC.estar.r1.ML<-T.r1[p]*log(sigma.r1[1,p])+(P[p,1]+R.star)*log(T.r1[p])
    BIC.estar.r1.LS<-T.r1[p]*log(sigma.r1[2,p])+(P[p,1]+R.star)*log(T.r1[p])
    AICc.estar.r1.ML<-T.r1[p]*log(sigma.r1[1,p])+(T.r1[p]+P[p,1]+R.star)/(1-(P[p,1]+2+R.star)/T.r1[p])
    AICc.estar.r1.LS<-T.r1[p]*log(sigma.r1[2,p])+(T.r1[p]+P[p,1]+R.star)/(1-(P[p,1]+2+R.star)/T.r1[p])
    WIC.estar.r1.ML <- T.r1[p]*log(sigma.r1[1,p])+((2*T.r1[p]*(P[p,1]+1+R.star)/(T.r1[p]-P[p,1]-2-R.star))^2+((P[p,1]+R.star)*log(T.r1[p]))^2)/((2*T.r1[p]*(P[p,1]+1+R.star)/(T.r1[p]-P[p,1]-2-R.star))+((P[p,1]+R.star)*log(T.r1[p])))
    WIC.estar.r1.LS <- T.r1[p]*log(sigma.r1[2,p])+((2*T.r1[p]*(P[p,1]+1+R.star)/(T.r1[p]-P[p,1]-2-R.star))^2+((P[p,1]+R.star)*log(T.r1[p]))^2)/((2*T.r1[p]*(P[p,1]+1+R.star)/(T.r1[p]-P[p,1]-2-R.star))+((P[p,1]+R.star)*log(T.r1[p])))

    # regime 2 criteria
    AIC.estar.r2.ML<-T.r2[p]*(log(sigma.r2[1,p])+1)+2*(P[p,2]+1+R.star)
    AIC.estar.r2.LS<-T.r2[p]*(log(sigma.r2[2,p])+1)+2*(P[p,2]+1+R.star)
    BIC.estar.r2.ML<-T.r2[p]*log(sigma.r2[1,p])+(P[p,2]+R.star)*log(T.r2[p])
    BIC.estar.r2.LS<-T.r2[p]*log(sigma.r2[2,p])+(P[p,2]+R.star)*log(T.r2[p])
    AICc.estar.r2.ML<-T.r2[p]*log(sigma.r2[1,p])+(T.r2[p]+P[p,2]+R.star)/(1-(P[p,2]+2+R.star)/T.r2[p])
    AICc.estar.r2.LS<-T.r2[p]*log(sigma.r2[2,p])+(T.r2[p]+P[p,2]+R.star)/(1-(P[p,2]+2+R.star)/T.r2[p])
    WIC.estar.r2.ML <- T.r2[p]*log(sigma.r2[1,p])+((2*T.r2[p]*(P[p,2]+1+R.star)/(T.r2[p]-P[p,2]-2-R.star))^2+((P[p,2]+R.star)*log(T.r2[p]))^2)/((2*T.r2[p]*(P[p,2]+1+R.star)/(T.r2[p]-P[p,2]-2-R.star))+((P[p,2]+R.star)*log(T.r2[p])))
    WIC.estar.r2.LS <- T.r2[p]*log(sigma.r2[2,p])+((2*T.r2[p]*(P[p,2]+1+R.star)/(T.r2[p]-P[p,2]-2-R.star))^2+((P[p,2]+R.star)*log(T.r2[p]))^2)/((2*T.r2[p]*(P[p,2]+1+R.star)/(T.r2[p]-P[p,2]-2-R.star))+((P[p,2]+R.star)*log(T.r2[p])))

    # overall criteria
    AIC.estar.ML.ov<-(n-p.max)*(log(sigma[1,p])+1)+2*(P[p,1]+P[p,2]+1+R.star)
    AIC.estar.OLS.ov<-(n-p.max)*(log(sigma[2,p])+1)+2*(P[p,1]+P[p,2]+1+R.star)
    BIC.estar.ML.ov<-(n-p.max)*log(sigma[1,p])+(P[p,1]+P[p,2]+R.star)*log(n-p.max)
    BIC.estar.OLS.ov<-(n-p.max)*log(sigma[2,p])+(P[p,1]+P[p,2]+R.star)*log(n-p.max)
    AICc.estar.ML.ov<-(n-p.max)*log(sigma[1,p])+(n-p.max+(P[p,1]+P[p,2]+R.star))/(1-((P[p,1]+P[p,2]+R.star)+2)/(n-p.max))
    AICc.estar.OLS.ov<-(n-p.max)*log(sigma[2,p])+(n-p.max+(P[p,1]+P[p,2]+R.star))/(1-((P[p,1]+P[p,2]+R.star)+2)/(n-p.max))
    WIC.estar.ML.ov <- (n-p.max)*log(sigma[1,p])+((2*(n-p.max)*((P[p,1]+P[p,2]+R.star)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.star)-2))^2+((P[p,1]+P[p,2]+R.star)*log(n-p.max))^2)/((2*(n-p.max)*((P[p,1]+P[p,2]+R.star)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.star)-2))+((P[p,1]+P[p,2]+R.star)*log(n-p.max)))
    WIC.estar.OLS.ov <-(n-p.max)*log(sigma[2,p])+((2*(n-p.max)*((P[p,1]+P[p,2]+R.star)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.star)-2))^2+((P[p,1]+P[p,2]+R.star)*log(n-p.max))^2)/((2*(n-p.max)*((P[p,1]+P[p,2]+R.star)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.star)-2))+((P[p,1]+P[p,2]+R.star)*log(n-p.max)))


    ### tabulate information criteria ###
    IC[q,1,p]<-sum(1-Trans.func[,p])/(n-p.max)*AIC.estar.r1.ML+sum(Trans.func[,p])/(n-p.max)*AIC.estar.r2.ML
    IC[q,2,p]<-sum(1-Trans.func[,p])/(n-p.max)*AIC.estar.r1.LS+sum(Trans.func[,p])/(n-p.max)*AIC.estar.r2.LS
    IC[q,3,p]<-sum(1-Trans.func[,p])/(n-p.max)*BIC.estar.r1.ML+sum(Trans.func[,p])/(n-p.max)*BIC.estar.r2.ML
    IC[q,4,p]<-sum(1-Trans.func[,p])/(n-p.max)*BIC.estar.r1.LS+sum(Trans.func[,p])/(n-p.max)*BIC.estar.r2.LS
    IC[q,5,p]<-sum(1-Trans.func[,p])/(n-p.max)*AICc.estar.r1.ML+sum(Trans.func[,p])/(n-p.max)*AICc.estar.r2.ML
    IC[q,6,p]<-sum(1-Trans.func[,p])/(n-p.max)*AICc.estar.r1.LS+sum(Trans.func[,p])/(n-p.max)*AICc.estar.r2.LS
    IC[q,7,p]<-sum(1-Trans.func[,p])/(n-p.max)*WIC.estar.r1.ML+sum(Trans.func[,p])/(n-p.max)*WIC.estar.r2.ML
    IC[q,8,p]<-sum(1-Trans.func[,p])/(n-p.max)*WIC.estar.r1.LS+sum(Trans.func[,p])/(n-p.max)*WIC.estar.r2.LS
    IC[q,9,p]<-(AIC.estar.r1.ML+AIC.estar.r2.ML)/2
    IC[q,10,p]<-(AIC.estar.r1.LS+AIC.estar.r2.LS)/2
    IC[q,11,p]<-(BIC.estar.r1.ML+BIC.estar.r2.ML)/2
    IC[q,12,p]<-(BIC.estar.r1.LS+BIC.estar.r2.LS)/2
    IC[q,13,p]<-(AICc.estar.r1.ML+AICc.estar.r2.ML)/2
    IC[q,14,p]<-(AICc.estar.r1.LS+AICc.estar.r2.LS)/2
    IC[q,15,p]<-(WIC.estar.r1.ML+WIC.estar.r2.ML)/2
    IC[q,16,p]<-(WIC.estar.r1.LS+WIC.estar.r2.LS)/2
    IC[q,17,p]<-AIC.estar.ML.ov
    IC[q,18,p]<-AIC.estar.OLS.ov
    IC[q,19,p]<-BIC.estar.ML.ov
    IC[q,20,p]<-BIC.estar.OLS.ov
    IC[q,21,p]<-AICc.estar.ML.ov
    IC[q,22,p]<-AICc.estar.OLS.ov
    IC[q,23,p]<-WIC.estar.ML.ov
    IC[q,24,p]<-WIC.estar.OLS.ov
} 

print(q)

}    
# write and save IC for each model
for (i in 1:p.row) {
    a <- data.frame(IC[, ,i])
    myfile <- file.path(paste0("IC.1000.AR(2).ESTAR", i, ".xls"))
    write.table(a, file = myfile, sep = '\t', row.names = FALSE, col.names = FALSE, quote = FALSE, append = FALSE)
}


zeit2<-Sys.time()
zeit2-zeit1
