### Clear Workspace ###
rm(list=ls(all=TRUE))
zeit1<-Sys.time()

### Load Data ###
# set working directory with setwd()
data <- read.table("AR(2).txt")

# Number of observations
n <-1000 # 100, 250, 500, 1000

# Number of Replications
Q <- ncol(data)

 ### maximum estimated model order
 p.max<-4
 p.row<-p.max^2
 P<-cbind(rep(1:p.max,each=p.max),rep(1:p.max,p.max))

 # Decision Matrices
IC<-array(NA,c(Q,24,p.row))    #row=Replication, col=Criterium, matrix=Model 

for(q in 1:Q)
{
 y <- data[(1:n),q]
 # effective sample size
 y.eff <- y[(p.max+1):n]

### SETAR Model ###

# additional parameters: threshold, delay
R.tar <- 2

#grid for threshold and delay parameter
kc <- seq(quantile(y,0.15),quantile(y,0.85),length=15)
kd <- 1:p.max

#lagged variables
lag.tar <-matrix(1:p.max,nrow=(n-p.max),ncol=p.max,byrow=TRUE)
Lag.tar <-as.vector(((p.max+1):n)-lag.tar)
x.tar   <- y[Lag.tar]
x.tar   <- matrix(x.tar,ncol=p.max)
T.r1    <- numeric(length(kc))
T.r2    <- numeric(length(kc))
RSS.r1  <- array(NA,c(length(kc),p.max,p.row)) #row=threshold ,col=delay ,matrix=model
RSS.r2  <- array(NA,c(length(kc),p.max,p.row)) #row=threshold ,col=delay ,matrix=model
RSS     <- array(NA,c(length(kc),p.max,p.row)) #row=threshold ,col=delay ,matrix=model
par.hat <- matrix(NA, nrow=p.row,ncol=2)       #1st col="threshold", 2nd col=delay
sigma.r1<- matrix(NA,nrow=2,ncol=p.row)
sigma.r2<- matrix(NA,nrow=2,ncol=p.row)
sigma   <- matrix(NA,nrow=2,ncol=p.row)

for(d in 1:p.max){
 for(k in 1:length(kc)){
  w <- numeric(n)
  for(i in (p.max+1):n){
   if(y[i-d]>kc[k]){w[i]<-1}
   }
   w <- w[(p.max+1):n]
   y.r1 <- y.eff[w==1]
   y.r2 <- y.eff[w==0]
   x.r1 <- x.tar[w==1,]
   x.r2 <- x.tar[w==0,]
   T.r1[k] <- length(y.r1)
   T.r2[k] <- length(y.r2)
   for(p in 1:p.row){
    RSS.r1[k,d,p]<-sum(summary(lm(y.r1~x.r1[,(1:P[p,1])]))$resid^2)
    RSS.r2[k,d,p]<-sum(summary(lm(y.r2~x.r2[,(1:P[p,2])]))$resid^2)
  }
 }
}

RSS <- RSS.r1 + RSS.r2      #row=threshold ,col=delay ,matrix=model

# estimated threshold and delay parameter for SETAR models
for(p in 1:p.row){
 if(nrow(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE))==1){
 par.hat[p,] <- which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)      #1st col="threshold", 2nd col=delay
 }else{
 par.hat[p,] <- c(round(mean(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)[,1])),min(which(RSS[,,p]==min(RSS[,,p]),arr.ind=TRUE)[,2]))
}
}
 T.r1 <- T.r1[par.hat[,1]]
 T.r2 <- T.r2[par.hat[,1]]



# estimates error term variances for SETAR models
for(p in 1:p.row){
sigma.r1[1,p] <- RSS.r1[par.hat[p,1],par.hat[p,2],p]/(T.r1[p])
sigma.r1[2,p] <- RSS.r1[par.hat[p,1],par.hat[p,2],p]/(T.r1[p]-P[p,1]-1)
sigma.r2[1,p] <- RSS.r2[par.hat[p,1],par.hat[p,2],p]/(T.r2[p])
sigma.r2[2,p] <- RSS.r2[par.hat[p,1],par.hat[p,2],p]/(T.r2[p]-P[p,2]-1)
sigma[1,p]    <- min(RSS[,,p])/(n-p.max)
sigma[2,p]    <- min(RSS[,,p])/(n-p.max-P[p,1]-P[p,2]-1)


    ### calculate information criteria of SETAR models ###
    # regime 1 criteria
    AIC.tar.r1.ML <- T.r1[p]*(log(sigma.r1[1,p])+1)+2*(P[p,1]+1+R.tar)
    AIC.tar.r1.LS <- T.r1[p]*(log(sigma.r1[2,p])+1)+2*(P[p,1]+1+R.tar)
    BIC.tar.r1.ML <- T.r1[p]*log(sigma.r1[1,p])+(P[p,1]+R.tar)*log(T.r1[p])
    BIC.tar.r1.LS <- T.r1[p]*log(sigma.r1[2,p])+(P[p,1]+R.tar)*log(T.r1[p])
    AICc.tar.r1.ML<- T.r1[p]*log(sigma.r1[1,p])+(T.r1[p]+P[p,1]+R.tar)/(1-(P[p,1]+2+R.tar)/T.r1[p])
    AICc.tar.r1.LS<- T.r1[p]*log(sigma.r1[2,p])+(T.r1[p]+P[p,1]+R.tar)/(1-(P[p,1]+2+R.tar)/T.r1[p])
    WIC.tar.r1.ML <- T.r1[p]*log(sigma.r1[1,p])+((2*T.r1[p]*(P[p,1]+1+R.tar)/(T.r1[p]-P[p,1]-2-R.tar))^2+((P[p,1]+R.tar)*log(T.r1[p]))^2)/((2*T.r1[p]*(P[p,1]+1+R.tar)/(T.r1[p]-P[p,1]-2-R.tar))+((P[p,1]+R.tar)*log(T.r1[p])))
    WIC.tar.r1.LS <- T.r1[p]*log(sigma.r1[2,p])+((2*T.r1[p]*(P[p,1]+1+R.tar)/(T.r1[p]-P[p,1]-2-R.tar))^2+((P[p,1]+R.tar)*log(T.r1[p]))^2)/((2*T.r1[p]*(P[p,1]+1+R.tar)/(T.r1[p]-P[p,1]-2-R.tar))+((P[p,1]+R.tar)*log(T.r1[p])))

    # regime 2 criteria
    AIC.tar.r2.ML <- T.r2[p]*(log(sigma.r2[1,p])+1)+2*(P[p,2]+1+R.tar)
    AIC.tar.r2.LS <- T.r2[p]*(log(sigma.r2[2,p])+1)+2*(P[p,2]+1+R.tar)
    BIC.tar.r2.ML <- T.r2[p]*log(sigma.r2[1,p])+(P[p,2]+R.tar)*log(T.r2[p])
    BIC.tar.r2.LS <- T.r2[p]*log(sigma.r2[2,p])+(P[p,2]+R.tar)*log(T.r2[p])
    AICc.tar.r2.ML<- T.r2[p]*log(sigma.r2[1,p])+(T.r2[p]+P[p,2]+R.tar)/(1-(P[p,2]+2+R.tar)/T.r2[p])
    AICc.tar.r2.LS<- T.r2[p]*log(sigma.r2[2,p])+(T.r2[p]+P[p,2]+R.tar)/(1-(P[p,2]+2+R.tar)/T.r2[p])
    WIC.tar.r2.ML <- T.r2[p]*log(sigma.r2[1,p])+((2*T.r2[p]*(P[p,2]+1+R.tar)/(T.r2[p]-P[p,2]-2-R.tar))^2+((P[p,2]+R.tar)*log(T.r2[p]))^2)/((2*T.r2[p]*(P[p,2]+1+R.tar)/(T.r2[p]-P[p,2]-2-R.tar))+((P[p,2]+R.tar)*log(T.r2[p])))
    WIC.tar.r2.LS <- T.r2[p]*log(sigma.r2[2,p])+((2*T.r2[p]*(P[p,2]+1+R.tar)/(T.r2[p]-P[p,2]-2-R.tar))^2+((P[p,2]+R.tar)*log(T.r2[p]))^2)/((2*T.r2[p]*(P[p,2]+1+R.tar)/(T.r2[p]-P[p,2]-2-R.tar))+((P[p,2]+R.tar)*log(T.r2[p])))

    # overall criteria
    AIC.tar.ML.ov <- (n-p.max)*(log(sigma[1,p])+1)+2*(P[p,1]+P[p,2]+1+R.tar)
    AIC.tar.LS.ov <- (n-p.max)*(log(sigma[2,p])+1)+2*(P[p,1]+P[p,2]+1+R.tar)
    BIC.tar.ML.ov <- (n-p.max)*log(sigma[1,p])+(P[p,1]+P[p,2]+R.tar)*log(n-p.max)
    BIC.tar.LS.ov <- (n-p.max)*log(sigma[2,p])+(P[p,1]+P[p,2]+R.tar)*log(n-p.max)
    AICc.tar.ML.ov<- (n-p.max)*log(sigma[1,p])+(n-p.max+(P[p,1]+P[p,2]+R.tar))/(1-((P[p,1]+P[p,2]+R.tar)+2)/(n-p.max))
    AICc.tar.LS.ov<- (n-p.max)*log(sigma[2,p])+(n-p.max+(P[p,1]+P[p,2]+R.tar))/(1-((P[p,1]+P[p,2]+R.tar)+2)/(n-p.max))
    WIC.tar.ML.ov <- (n-p.max)*log(sigma[1,p])+((2*(n-p.max)*((P[p,1]+P[p,2]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.tar)-2))^2+((P[p,1]+P[p,2]+R.tar)*log(n-p.max))^2)/((2*(n-p.max)*((P[p,1]+P[p,2]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.tar)-2))+((P[p,1]+P[p,2]+R.tar)*log(n-p.max)))
    WIC.tar.LS.ov <- (n-p.max)*log(sigma[2,p])+((2*(n-p.max)*((P[p,1]+P[p,2]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.tar)-2))^2+((P[p,1]+P[p,2]+R.tar)*log(n-p.max))^2)/((2*(n-p.max)*((P[p,1]+P[p,2]+R.tar)+1)/((n-p.max)-(P[p,1]+P[p,2]+R.tar)-2))+((P[p,1]+P[p,2]+R.tar)*log(n-p.max)))
####
    ### tabulate information criteria ###
    IC[q,1,p]<-T.r1[p]/(n-p.max)*AIC.tar.r1.ML+T.r2[p]/(n-p.max)*AIC.tar.r2.ML
    IC[q,2,p]<-T.r1[p]/(n-p.max)*AIC.tar.r1.LS+T.r2[p]/(n-p.max)*AIC.tar.r2.LS
    IC[q,3,p]<-T.r1[p]/(n-p.max)*BIC.tar.r1.ML+T.r2[p]/(n-p.max)*BIC.tar.r2.ML
    IC[q,4,p]<-T.r1[p]/(n-p.max)*BIC.tar.r1.LS+T.r2[p]/(n-p.max)*BIC.tar.r2.LS
    IC[q,5,p]<-T.r1[p]/(n-p.max)*AICc.tar.r1.ML+T.r2[p]/(n-p.max)*AICc.tar.r2.ML
    IC[q,6,p]<-T.r1[p]/(n-p.max)*AICc.tar.r1.LS+T.r2[p]/(n-p.max)*AICc.tar.r2.LS
    IC[q,7,p]<-T.r1[p]/(n-p.max)*WIC.tar.r1.ML+T.r2[p]/(n-p.max)*WIC.tar.r2.ML
    IC[q,8,p]<-T.r1[p]/(n-p.max)*WIC.tar.r1.LS+T.r2[p]/(n-p.max)*WIC.tar.r2.LS
    IC[q,9,p]<-(AIC.tar.r1.ML+AIC.tar.r2.ML)/2
    IC[q,10,p]<-(AIC.tar.r1.LS+AIC.tar.r2.LS)/2
    IC[q,11,p]<-(BIC.tar.r1.ML+BIC.tar.r2.ML)/2
    IC[q,12,p]<-(BIC.tar.r1.LS+BIC.tar.r2.LS)/2
    IC[q,13,p]<-(AICc.tar.r1.ML+AICc.tar.r2.ML)/2
    IC[q,14,p]<-(AICc.tar.r1.LS+AICc.tar.r2.LS)/2
    IC[q,15,p]<-(WIC.tar.r1.ML+WIC.tar.r2.ML)/2
    IC[q,16,p]<-(WIC.tar.r1.LS+WIC.tar.r2.LS)/2
    IC[q,17,p]<-AIC.tar.ML.ov
    IC[q,18,p]<-AIC.tar.LS.ov
    IC[q,19,p]<-BIC.tar.ML.ov
    IC[q,20,p]<-BIC.tar.LS.ov
    IC[q,21,p]<-AICc.tar.ML.ov
    IC[q,22,p]<-AICc.tar.LS.ov
    IC[q,23,p]<-WIC.tar.ML.ov
    IC[q,24,p]<-WIC.tar.LS.ov
} 
print(q)
}    

# write and save IC for each model
for (i in 1:p.row) {
    a <- data.frame(IC[, ,i])
    myfile <- file.path(paste0("IC.1000.AR(2).TAR", i, ".xls"))
    write.table(a, file = myfile, sep = '\t', row.names = FALSE, col.names = FALSE, quote = FALSE, append = FALSE)
}


zeit2<-Sys.time()
zeit2-zeit1


